#!/usr/bin/env python
# coding=utf-8
import time
import datetime
from collections import deque

class MessageStat(dict):

    def __init__(self):
        self.online = 0
        self.auth_req_old = 0
        self.auth_resp_old = 0
        self.auth_req = 0
        self.auth_accept = 0
        self.auth_reject = 0
        self.auth_drop = 0
        self.acct_start = 0
        self.acct_stop = 0
        self.acct_update = 0
        self.acct_on = 0
        self.acct_off = 0
        self.acct_req_old = 0
        self.acct_resp_old = 0
        self.acct_req = 0
        self.acct_resp = 0
        self.acct_retry = 0
        self.acct_drop = 0
        self.auth_req_stat = deque([],90)
        self.auth_resp_stat = deque([],90)
        self.acct_req_stat = deque([],90)
        self.acct_resp_stat = deque([],90)
        self.last_max_req = 0
        self.last_max_req_date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.last_max_resp = 0
        self.last_max_resp_date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    def incr(self, attr_name, incr=1):
        if hasattr(self, attr_name):
            setattr(self, attr_name, getattr(self,attr_name) + incr)
        
    def run_stat(self):
        _time = time.time()*1000
        _auth_req_stat = self.auth_req - self.auth_req_old
        self.auth_req_old = self.auth_req

        _auth_resp_stat = (self.auth_accept+self.auth_reject) - self.auth_resp_old
        self.auth_resp_old =  (self.auth_accept+self.auth_reject) 

        _acct_req_stat = self.acct_req - self.acct_req_old
        self.acct_req_old = self.acct_req

        _acct_resp_stat = self.acct_resp - self.acct_resp_old
        self.acct_resp_old = self.acct_resp

        self.auth_req_stat.append((_time,_auth_req_stat))
        self.auth_resp_stat.append((_time,_auth_resp_stat))
        self.acct_req_stat.append((_time,_acct_req_stat))
        self.acct_resp_stat.append((_time,_acct_resp_stat))

        req_percount = int((_auth_req_stat+_acct_req_stat)/10.0)
        if self.last_max_req < req_percount:
            self.last_max_req = req_percount
            self.last_max_req_date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        resp_percount = int((_auth_resp_stat+_acct_resp_stat)/10.0)
        if self.last_max_resp < resp_percount:
            self.last_max_resp = resp_percount
            self.last_max_resp_date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    def __getattr__(self, key): 
        try:
            return self[key]
        except KeyError, k:
            raise AttributeError, k
    
    def __setattr__(self, key, value): 
        self[key] = value
    
    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError, k:
            raise AttributeError, k        



#!/usr/bin/env python
# coding=utf-8


import des
import md4
import hashlib
import utils


def challenge_hash(peer_challenge, authenticator_challenge, username):
    """ChallengeHash"""
    sha_hash = hashlib.sha1()
    sha_hash.update(peer_challenge)
    sha_hash.update(authenticator_challenge)
    sha_hash.update(username)
    return sha_hash.digest()[:8]


def nt_password_hash(passwd):
    """NtPasswordHash"""
    pw = utils.str2unicode(passwd)
    md4_context = md4.new()
    md4_context.update(pw)
    return md4_context.digest()


def hash_nt_password_hash(password_hash):
    """HashNtPasswordHash"""
    md4_context = md4.new()
    md4_context.update(password_hash)
    return md4_context.digest()


def generate_nt_response_mschap(challenge, password):
    password_hash = nt_password_hash(password)
    return challenge_response(challenge, password_hash)


def generate_nt_response_mschap2(authenticator_challenge, peer_challenge, username, password):
    """GenerateNTResponse"""
    challenge = challenge_hash(peer_challenge, authenticator_challenge, username)
    password_hash = nt_password_hash(password)
    return challenge_response(challenge, password_hash)



def challenge_response(challenge, password_hash):
    """ChallengeResponse"""
    zpassword_hash = password_hash.ljust(21, '\0')

    response = ""
    des_obj = des.DES(zpassword_hash[0:7])
    response += des_obj.encrypt(challenge)

    des_obj = des.DES(zpassword_hash[7:14])
    response += des_obj.encrypt(challenge)

    des_obj = des.DES(zpassword_hash[14:21])
    response += des_obj.encrypt(challenge)
    return response


def generate_authenticator_response(password, nt_response, peer_challenge, authenticator_challenge, username):
    """GenerateAuthenticatorResponse"""
    Magic1 = "\x4D\x61\x67\x69\x63\x20\x73\x65\x72\x76\x65\x72\x20\x74\x6F\x20\x63\x6C\x69\x65\x6E\x74\x20\x73\x69\x67\x6E\x69\x6E\x67\x20\x63\x6F\x6E\x73\x74\x61\x6E\x74"
    Magic2 = "\x50\x61\x64\x20\x74\x6F\x20\x6D\x61\x6B\x65\x20\x69\x74\x20\x64\x6F\x20\x6D\x6F\x72\x65\x20\x74\x68\x61\x6E\x20\x6F\x6E\x65\x20\x69\x74\x65\x72\x61\x74\x69\x6F\x6E"
    password_hash = nt_password_hash(password)
    password_hash_hash = hash_nt_password_hash(password_hash)

    sha_hash = hashlib.sha1()
    sha_hash.update(password_hash_hash)
    sha_hash.update(nt_response)
    sha_hash.update(Magic1)
    digest = sha_hash.digest()

    challenge = challenge_hash(peer_challenge, authenticator_challenge, username)

    sha_hash = hashlib.sha1()
    sha_hash.update(digest)
    sha_hash.update(challenge)
    sha_hash.update(Magic2)
    digest = sha_hash.digest()

    return "\x01S=" + convert_to_hex_string(digest)


def check_authenticator_response(password, nt_response, peer_challenge, authenticator_challenge, user_name, received_response):
    """CheckAuthenticatorResponse"""
    my_resppnse = generate_authenticator_response(password, nt_response, peer_challenge, authenticator_challenge, user_name)

    return my_resppnse == received_response

def convert_to_hex_string(string):
    hex_str = ""
    for c in string:
        hex_tmp = hex(ord(c))[2:]
        if len(hex_tmp) == 1:
            hex_tmp = "0" + hex_tmp
        hex_str += hex_tmp
    return hex_str.upper()





def lm_password_hash(password):

    ucase_password = password.upper()[:14]
    while len(ucase_password) < 14:
        ucase_password += "\0"
    password_hash = des_hash(ucase_password[:7])
    password_hash += des_hash(ucase_password[7:])
    return password_hash


def des_hash(clear):
    """DesEncrypt"""
    des_obj = des.DES(clear)
    return des_obj.encrypt(r"KGS!@#$%")




#!/usr/bin/env python
#coding=utf-8

#  vlan parse          
def parse_cisco(req):
    '''phy_slot/phy_subslot/phy_port:XPI.XCI'''
    nasportid = req.get('NAS-Port-Id')
    if not nasportid:return
    nasportid = nasportid.lower()
    def parse_vlanid():
        ind = nasportid.find(':')
        if ind == -1:return
        ind2 = nasportid.find('.',ind)
        if ind2 == -1:
            req.vlanid = int(nasportid[ind+1])
        else:
            req.vlanid = int(nasportid[ind+1:ind2])
    def parse_vlanid2():
        ind = nasportid.find('.')
        if ind == -1:return
        ind2 = nasportid.find(' ',ind)
        if ind2 == -1:
            req.vlanid2 = int(nasportid[ind+1])
        else:
            req.vlanid2 = int(nasportid[ind+1:ind2])
            
    parse_vlanid()
    parse_vlanid2()
    return req


def parse_std(req):
    ''''''
    nasportid = req.get('NAS-Port-Id')
    if not nasportid:return
    nasportid = nasportid.lower()
    def parse_vlanid():
        ind = nasportid.find('vlanid=')
        if ind == -1:return
        ind2 = nasportid.find(';',ind)
        if ind2 == -1:
            req.vlanid = int(nasportid[ind+7])
        else:
            req.vlanid = int(nasportid[ind+7:ind2])
            
    def parse_vlanid2():
        ind = nasportid.find('vlanid2=')
        if ind == -1:return
        ind2 = nasportid.find(';',ind)
        if ind2 == -1:
            req.vlanid2 = int(nasportid[ind+8])
        else:
            req.vlanid2 = int(nasportid[ind+8:ind2])
            
    parse_vlanid()
    parse_vlanid2() 
    return req

def parse_ros(req):
    ''''''
    nasportid = req.get('NAS-Port-Id')
    if not nasportid:return
    nasportid = nasportid.lower()
    def parse_vlanid():
        ind = nasportid.find(':')
        if ind == -1:return        
        ind2 = nasportid.find(' ',ind)
        if ind2 == -1:return
        req.vlanid = int(nasportid[ind+1:ind2])
        
    def parse_vlanid2():
        ind = nasportid.find(':')
        if ind == -1:return
        ind2 = nasportid.find('.',ind)
        if ind2 == -1:return
        req.vlanid2 = int(nasportid[ind+1:ind2])   
    parse_vlanid()
    parse_vlanid2()  
    return req             
  
parse_radback = parse_ros
parse_zte = parse_ros

_parses = {
    '0' : parse_std,
    '9' : parse_cisco,
    '3041' : parse_cisco,
    '2352' : parse_radback,
    '2011' : parse_std,
    '25506' : parse_std,
    '3902' : parse_zte,
    '14988' : parse_ros
}

def process(req):
    if req.vendor_id in _parses:
        _parses[req.vendor_id](req)
    return req




#!/usr/bin/env python
#coding=utf-8
from txradius.radius.packet import tools
from txradius.radius.packet import AuthPacket
from txradius.radius.packet import AcctPacket
from txradius.radius.packet import CoAPacket
from txradius.radius.packet import AccessRequest
from txradius.radius.packet import AccessAccept
from txradius.radius.packet import AccountingRequest
from txradius.radius.packet import AccountingResponse
from txradius.radius.packet import CoARequest
from txradius.mschap import mschap,mppe
import time
import binascii
import datetime
import hashlib
import six
import decimal

decimal.getcontext().prec = 16
decimal.getcontext().rounding = decimal.ROUND_UP


md5_constructor = hashlib.md5

PacketStatusTypeMap = {
    1 : 'AccessRequest',
    2 : 'AccessAccept',
    3 : 'AccessReject',
    4 : 'AccountingRequest',
    5 : 'AccountingResponse',
    40 : 'DisconnectRequest',
    41 : 'DisconnectACK',
    42 : 'DisconnectNAK',
    43 : 'CoARequest',
    44 : 'CoAACK',
    45 : 'CoANAK',
}

def format_packet_str(pkt):
    attr_keys = pkt.keys()
    _str = "\nRadius Packet:%s"%PacketStatusTypeMap.get(pkt.code)
    _str += "\nid:%s" % pkt.id
    _str += "\ncode:%s" % pkt.code
    _str += "\nauth:%s" % [pkt.authenticator]
    _str += "\nAttributes: "     
    for attr in attr_keys:
        try:
            _str += "\n\t%s: %s " % (attr, pkt[attr])   
        except:
            pass
    return _str


def format_packet_log(pkt):
    attr_keys = pkt.keys()
    _str = "RadiusPacket:%s;" % PacketStatusTypeMap[pkt.code]
    _str += "id:%s;" % pkt.id
    _str += "code:%s;" % pkt.code
    _str += "auth:%s" % [pkt.authenticator]
    for attr in attr_keys:
        try:
            _str += "%s:%s;" % (attr, pkt[attr])
        except:
            pass
    return _str


def get_session_timeout(pkt,defval=86400):
    try:return tools.DecodeInteger(pkt.get(27)[0]) or defval
    except:return defval

def get_interim_update(pkt,defval=300):
    try:return tools.DecodeInteger(pkt.get(85)[0]) or defval
    except:return defval

class ExtAttrMixin:

    def __init__(self):
        self._created = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self._vendor_id = 0
        self._vlanid1 = 0
        self._vlanid2 = 0
        self._client_mac = None
        self.resp_attrs = {}

    @property
    def vendor_id(self):
        return self._vendor_id

    @vendor_id.setter
    def vendor_id(self, vendor_id):
        self._vendor_id = vendor_id

    @property
    def vlanid1(self):
        return self._vlanid1

    @vlanid1.setter
    def vlanid1(self,vlanid1):
        self._vlanid1 = vlanid1

    @property
    def vlanid2(self):
        return _vlanid2

    @vendor_id.setter
    def vlanid2(self,vlanid2):
        self._vlanid2 = vlanid2

    @property
    def client_mac(self):
        return _client_mac

    @vendor_id.setter
    def client_mac(self,macaddr):
        self._client_mac = macaddr

    def get_vlanids(self):
        return self.get_vlanid1(),self.get_vlanid2()  

    @property
    def created(self):
        return self._created



class CoAMessage(CoAPacket,ExtAttrMixin):
    def __init__(self, code=CoARequest, id=None, secret=six.b(''),
            authenticator=None, **attributes):
        CoAPacket.__init__(self, code, id, secret, authenticator, **attributes)
        ExtAttrMixin.__init__(self)
        
    def format_str(self):
        return format_packet_str(self)    

    def format_log(self):
        return format_packet_log(self)

    def get_acct_sessionid(self):
        try:return tools.DecodeString(self.get(44)[0])
        except:pass    

    def get_framed_ipaddr(self):
        try:return tools.DecodeAddress(self.get(8)[0])
        except:pass

    def get_nas_addr(self):
        try:
            return tools.DecodeAddress(self.get(4)[0])
        except:pass

class AuthMessage(AuthPacket,ExtAttrMixin):

    def __init__(self, code=AccessRequest, id=None, secret=six.b(''), authenticator=None, **attributes):
        AuthPacket.__init__(self, code, id, secret, authenticator, **attributes)
        ExtAttrMixin.__init__(self)

    def format_str(self):
        return format_packet_str(self)

    def format_log(self):
        return format_packet_log(self)

    def __str__(self):
        _str = PacketStatusTypeMap[self.code]
        _str += ",id=%s"%self.id
        if self.code == 1:
            _str += ",username=%s,mac_addr=%s" % (self.get_user_name(),self.get_mac_addr())
        if 'Reply-Message' in self:
            _str += ',Reply-Message="%s"' % self['Reply-Message'][0]
        return _str   

    def CreateReply(self, **attributes):
        return AuthMessage(AccessAccept, self.id,
            self.secret, self.authenticator, dict=self.dict,
            **attributes)
        
    def ChapEcrypt(self,password):
        if not self.authenticator:
            self.authenticator = self.CreateAuthenticator()
        if not self.id:
            self.id = self.CreateID()
        if isinstance(password, six.text_type):
            password = password.strip().encode('utf-8')

        chapid = self.authenticator[0]
        self['CHAP-Challenge'] = self.authenticator
        return '%s%s' % (chapid, md5_constructor("%s%s%s" % (chapid, password, self.authenticator)).digest())

   
    def get_nas_addr(self):
        _nas_addr = None
        try:
            return tools.DecodeAddress(self.get(4)[0])
        except:pass
        
    def get_mac_addr(self):
        try:
            return self.get_extattr(self.client_mac,tools.DecodeString(self.get(31)[0]).replace("-",":")) 
        except:return None

    def get_user_name(self):
        try:
            user_name = tools.DecodeString(self.get(1)[0])
            if "@" in user_name:
                user_name = user_name[:user_name.index("@")]
            return user_name
        except:
            return None

    def get_domain(self):
        try:
            user_name = tools.DecodeString(self.get(1)[0])
            if "@" in user_name:
                return user_name[user_name.index("@")+1:]
        except:
            return None            
        

    def get_passwd(self):
        try:return self.PwDecrypt(self.get(2)[0])
        except:return None        

    def get_chappwd(self):
        try:return tools.DecodeOctets(self.get(3)[0])
        except:return None  
        
    def verifyChapEcrypt(self,userpwd):
        if isinstance(userpwd, six.text_type):
            userpwd = userpwd.strip().encode('utf-8')   

        _password = self.get_chappwd()
        if len(_password) != 17:
            return False

        chapid = _password[0]
        password = _password[1:]

        if not self.authenticator:
            self.authenticator = self.CreateAuthenticator()

        challenge = self.authenticator
        if 'CHAP-Challenge' in self:
            challenge = self['CHAP-Challenge'][0] 

        _pwd =  md5_constructor("%s%s%s"%(chapid,userpwd,challenge)).digest()
        return password == _pwd
        

    def verifyMsChapV2(self,userpwd):
        ms_chap_response = self['MS-CHAP2-Response'][0]
        authenticator_challenge = self['MS-CHAP-Challenge'][0]
        if len(ms_chap_response)!=50:
            raise Exception("Invalid MSCHAPV2-Response attribute length")
        # if isinstance(userpwd, six.text_type):
        #     userpwd = userpwd.strip().encode('utf-8')
        
        nt_response = ms_chap_response[26:50]
        peer_challenge = ms_chap_response[2:18]
        _user_name = self.get(1)[0]
        nt_resp = mschap.generate_nt_response_mschap2(
            authenticator_challenge,
            peer_challenge,
            _user_name,
            userpwd,
        )
        if nt_resp == nt_response:
            auth_resp = mschap.generate_authenticator_response(
                userpwd,
                nt_response,
                peer_challenge,
                authenticator_challenge,
                _user_name
            )
            self.resp_attrs['MS-CHAP2-Success'] = auth_resp
            self.resp_attrs['MS-MPPE-Encryption-Policy'] = '\x00\x00\x00\x01'
            self.resp_attrs['MS-MPPE-Encryption-Type'] = '\x00\x00\x00\x06'
            mppeSendKey, mppeRecvKey = mppe.mppe_chap2_gen_keys(userpwd, nt_response)
            send_key, recv_key = mppe.gen_radius_encrypt_keys(
                mppeSendKey,
                mppeRecvKey,
                self.secret,
                self.authenticator)
            self.resp_attrs['MS-MPPE-Send-Key'] = send_key
            self.resp_attrs['MS-MPPE-Recv-Key'] = recv_key
            return True
        else:
            self.resp_attrs['Reply-Message'] = "E=691 R=1 C=%s V=3 M=<password error>" % ('\0' * 32)
            return False
        
        
    def get_pwd_type(self):
        if 'MS-CHAP-Challenge' in self:
            if 'MS-CHAP-Response' in self:
                return 'mschapv1'
            elif 'MS-CHAP2-Response' in self:
                return 'mschapv2'
        elif 'CHAP-Password' in self:
            return 'chap'
        else:
            return 'pap'
            

    def is_valid_pwd(self,userpwd):
        pwd_type = self.get_pwd_type()
        try:
            if pwd_type == 'pap':
                return userpwd == self.get_passwd()
            elif pwd_type == 'chap':
                return self.verifyChapEcrypt(userpwd)
            elif pwd_type == 'mschapv1':
                return False
            elif pwd_type == 'mschapv2':
                return self.verifyMsChapV2(userpwd)
            else:
                return False
        except Exception as err:
            import traceback
            traceback.print_exc()
            return False

    def PwDecrypt(self, password):
        buf = password
        pw = six.b('')

        last = self.authenticator
        while buf:
            hash = md5_constructor(self.secret + last).digest()
            for i in range(16):
                pw += chr(ord(hash[i]) ^ ord(buf[i]))

            (last, buf) = (buf[:16], buf[16:])

        while pw.endswith(six.b('\x00')):
            pw = pw[:-1]

        return pw.decode('utf-8')

    def PwCrypt(self, password):
        if self.authenticator is None:
            self.authenticator = self.CreateAuthenticator()

        if isinstance(password, six.text_type):
            password = password.encode('utf-8')

        buf = password
        if len(password) % 16 != 0:
            buf += six.b('\x00') * (16 - (len(password) % 16))

        hash = md5_constructor(self.secret + self.authenticator).digest()
        result = six.b('')

        last = self.authenticator
        while buf:
            hash = md5_constructor(self.secret + last).digest()
            for i in range(16):
                result += chr(ord(hash[i]) ^ ord(buf[i]))

            last = result[-16:]
            buf = buf[16:]

        return result





class AcctMessage(AcctPacket,ExtAttrMixin):
    def __init__(self, code=AccountingRequest, id=None, secret=six.b(''),
            authenticator=None, **attributes):
        AcctPacket.__init__(self, code, id, secret, authenticator, **attributes)
        ExtAttrMixin.__init__(self)

    def format_str(self):
        return format_packet_str(self)

    def format_log(self):
        return format_packet_log(self)

    def __str__(self):
        _str = PacketStatusTypeMap.get(self.code)
        _str += ",id=%s"%self.id
        if self.code == 4:
            _str += ",username=%s,mac_addr=%s" % (self.get_user_name(),self.get_mac_addr())
        return _str   

    def CreateReply(self,**attributes):
        return AcctMessage(AccountingResponse, self.id,
            self.secret, self.authenticator, dict=self.dict,
            **attributes)    

    def get_user_name(self):
        try:
            user_name = tools.DecodeString(self.get(1)[0])
            if "@" in user_name:
                return user_name[:user_name.index("@")]
            else:
                return user_name
        except:
            return None
 

    def get_mac_addr(self):
        try:
            return self.client_mac
        except:return None

    def get_nas_addr(self):
        _nas_addr = None
        try:
            return tools.DecodeAddress(self.get(4)[0])
        except:pass


    def get_nas_port(self):
        try:return tools.DecodeInteger(self.get(5)[0]) or 0
        except:return 0

    def get_service_type(self):
        try:return tools.DecodeInteger(self.get(0)[0]) or 0
        except:return 0
        
    def get_framed_ipaddr(self):
        try:return tools.DecodeAddress(self.get(8)[0])
        except:return None

    def get_framed_netmask(self):
        try:return tools.DecodeAddress(self.get(9)[0])
        except:return None

    def get_nas_class(self):
        try:return tools.DecodeString(self.get(25)[0])
        except:return None   

    def get_session_timeout(self):
        try:return tools.DecodeInteger(self.get(27)[0]) or 0
        except:return 0

    def get_calling_stationid(self):
        try:return tools.DecodeString(self.get(31)[0])
        except:return None   

    def get_acct_status_type(self):
        try:return tools.DecodeInteger(self.get(40)[0])
        except:return None

    def get_acct_input_octets(self):
        try:return tools.DecodeInteger(self.get(42)[0]) or 0
        except:return 0

    def get_acct_output_octets(self):
        try:return tools.DecodeInteger(self.get(43)[0]) or 0
        except:return 0

    def get_acct_sessionid(self):
        try:return tools.DecodeString(self.get(44)[0])
        except:return None                                                         

    def get_acct_sessiontime(self):
        try:return tools.DecodeInteger(self.get(46)[0]) or 0
        except:return 0                                                             

    def get_acct_input_packets(self):
        try:return tools.DecodeInteger(self.get(47)[0]) or 0
        except:return 0                                                       

    def get_acct_output_packets(self):
        try:return tools.DecodeInteger(self.get(48)[0]) or 0
        except:return 0           

    def get_acct_terminate_cause(self):
        try:return tools.DecodeInteger(self.get(49)[0]) or 0
        except:return 0           

    def get_acct_input_gigawords(self):
        try:return tools.DecodeInteger(self.get(52)[0]) or 0
        except:return 0       

    def get_acct_output_gigawords(self):
        try:return tools.DecodeInteger(self.get(53)[0]) or 0
        except:return 0    

    def get_input_total(self):
        bl = decimal.Decimal(self.get_acct_input_octets())/decimal.Decimal(1024)
        gl = decimal.Decimal(self.get_acct_input_gigawords())*decimal.Decimal(4*1024*1024)
        tl = bl + gl
        return int(tl.to_integral_value())   
        
    def get_output_total(self):
        bl = decimal.Decimal(self.get_acct_output_octets())/decimal.Decimal(1024)
        gl = decimal.Decimal(self.get_acct_output_gigawords())*decimal.Decimal(4*1024*1024)
        tl = bl + gl
        return int(tl.to_integral_value())                                                            

    def get_event_timestamp(self):
        try:
            return tools.DecodeDate(self.get(55)[0])
        except:
            return None

    def get_event_timestamp_str(self,timetype=0):
        try:
            _time = tools.DecodeDate(self.get(55)[0])
            if timetype == 0:
                return datetime.datetime.fromtimestamp(_time).strftime("%Y-%m-%d %H:%M:%S")
            else:
                return datetime.datetime.fromtimestamp(_time-(8*3600)).strftime("%Y-%m-%d %H:%M:%S")
        except:
            return None

    def get_nas_port_type(self):
        try:return tools.DecodeInteger(self.get(61)[0]) or 0
        except:return 0   

    def get_nas_portid(self):
        try:return tools.DecodeString(self.get(87)[0])
        except:return None    

    def get_ticket(self):
        return dict(
            account_number = self.get_user_name(),
            mac_addr = self.get_mac_addr(),
            nas_addr = self.get_nas_addr(),
            nas_port = self.get_nas_port(),
            service_type = self.get_service_type(),
            framed_ipaddr = self.get_framed_ipaddr(),
            framed_netmask = self.get_framed_netmask(),
            nas_class = self.get_nas_class(),
            session_timeout = self.get_session_timeout(),
            calling_station_id = self.get_calling_stationid(),
            acct_status_type = self.get_acct_status_type(),
            acct_input_octets = self.get_acct_input_octets(),
            acct_output_octets = self.get_acct_output_octets(),
            acct_session_id = self.get_acct_sessionid(),
            acct_session_time = self.get_acct_sessiontime(),
            acct_input_packets = self.get_acct_input_packets(),
            acct_output_packets = self.get_acct_output_packets(),
            acct_terminate_cause = self.get_acct_terminate_cause(),
            acct_input_gigawords = self.get_acct_input_gigawords(),
            acct_output_gigawords = self.get_acct_output_gigawords(),
            event_timestamp = self.get_event_timestamp_str(),
            nas_port_type=self.get_nas_port_type(),
            nas_port_id=self.get_nas_portid()
        )
        
 


#!/usr/bin/env python
#coding=utf-8


def std_rate(resp, _in, _out):
    # input_limit = str(_in)
    # output_limit = str(_out)
    # _class = input_limit.zfill(8) + input_limit.zfill(8) + output_limit.zfill(8) + output_limit.zfill(8)
    # resp['Class'] = _class
    return resp


def ros_rate(resp, _in, _out):
    _irate = _in / 1024
    _orate = _out / 1024
    resp['Mikrotik-Rate-Limit'] = '%sk/%sk' % (_irate, _orate)
    return resp


def aikuai_rate(resp, _in, _out):
    _irate = _in / 1024 / 8
    _orate = _out / 1024 / 8
    resp['RP-Upstream-Speed-Limit'] = _irate
    resp['RP-Downstream-Speed-Limit'] = _orate
    return resp


def cisco_rate(resp, _in, _out):
    return resp


def radback_rate(resp, _in, _out):
    return resp


def h3c_rate(resp, _in, _out):
    resp = std_rate(resp, _in, _out)
    resp['H3C-Input-Average-Rate'] = _in
    resp['H3C-Input-Peak-Rate'] = _in
    resp['H3C-Output-Average-Rate'] = _out
    resp['H3C-Output-Peak-Rate'] = _out
    return resp


def zte_rate(resp, _in, _out):
    resp['ZTE-Rate-Ctrl-Scr-Up'] = _in 
    resp['ZTE-Rate-Ctrl-Scr-Down'] = _out 
    return resp


def huawei_rate(resp, _in, _out):
    input_limit = str(_in)
    output_limit = str(_out)
    _class = input_limit.zfill(8) + input_limit.zfill(8) + output_limit.zfill(8) + output_limit.zfill(8)
    resp['Class'] = _class
    resp['Huawei-Input-Average-Rate'] = _in
    resp['Huawei-Input-Peak-Rate'] = _in
    resp['Huawei-Output-Average-Rate'] = _out
    resp['Huawei-Output-Peak-Rate'] = _out
    return resp


rate_funcs = {
    '0': std_rate,
    '9': cisco_rate,
    '2011': huawei_rate,
    '2352': radback_rate,
    '3902': zte_rate,
    '25506': h3c_rate,
    '14988': ros_rate,
    '10055': aikuai_rate
}


def process(resp=None, input_rate=0, output_rate=0):
    if input_rate == 0 and output_rate == 0:
        return std_rate(resp, 0, 0)

    _vendor = resp.vendor_id or '0'
    return rate_funcs[_vendor](resp, input_rate, output_rate)


#!/usr/bin/env python
#coding=utf-8
import struct
from txradius.radius import  tools

IK_RAD_PKG_VER            = 0x0001
IK_RAD_PKG_AUTH           = 0x0001
IK_RAD_PKG_USR_PWD_TAG    = 0x0011
IK_RAD_PKG_USR_NAME_TAG   = 0x0012
IK_RAD_PKG_USR_CMD_TAG    = 0x0013
IK_RAD_PKG_CMD_ARGS_TAG   = 0x0014
IK_RAD_PKG_USR_CMD_LEN    = 4
IK_RAD_MAX_TLV            = 10
IK_RAD_STRING_MAX_LENGTH  = 32 
IK_STR_MEM_LEN            = 40

VENDOR_ID = 10055


def create_dm_pkg(secret,username):
    ''' create ikuai dm message'''
    secret = tools.EncodeString(secret)
    username = tools.EncodeString(username)
    pkg_format = '>HHHH32sHH32s'
    pkg_vals = [
        IK_RAD_PKG_VER,
        IK_RAD_PKG_AUTH,
        IK_RAD_PKG_USR_PWD_TAG,
        len(secret),
        secret.ljust(32,'\x00'),
        IK_RAD_PKG_CMD_ARGS_TAG,
        len(username),
        username.ljust(32,'\x00')
    ]
    return struct.pack(pkg_format,*pkg_vals)

if __name__ == '__main__':
    pp = create_dm_pkg('123456', 'testuser')
    print repr(pp)


import sys,os
sys.path.insert(0, "../../")
from txradius import message, client
from txradius.radius import dictionary,packet
from twisted.internet import reactor, defer
from twisted.python import log
import functools

log.startLogging(sys.stderr)



@defer.inlineCallbacks
def test_auth():
    _dict =dictionary.Dictionary('../../dictionary/dictionary')
    sender = functools.partial(client.send_auth,'secret',_dict,"127.0.0.1")

    req = {}
    req['User-Name'] = 'test01'
    req['CHAP-Password'] = '888888'
    req["NAS-IP-Address"]     = "192.168.1.10"
    req["NAS-Port"]           = 0
    req["Service-Type"]       = "Login-User"
    req["NAS-Identifier"]     = "trillian"
    req["Called-Station-Id"]  = "00-04-5F-00-0F-D1"
    req["Calling-Station-Id"] = "00-01-24-80-B3-9C"
    req["Framed-IP-Address"]  = "10.0.0.100"
    log.msg("req:"+ repr(req))
    resp = yield sender(**req)
    log.msg('auth done %s'% repr(resp))
    reactor.stop()

reactor.callLater(0.1, test_auth,)
reactor.run()








#!/usr/bin/env python
# coding=utf-8

from twisted.python import log
from twisted.internet import protocol
from twisted.internet import reactor, defer
from txradius.radius import packet
from txradius import message
import six
import time

class RadiusClient(protocol.DatagramProtocol):
    def __init__(self, secret, dictionary, server, authport=1812, acctport=1813,  debug=False):
        self.dict = dictionary
        self.secret = six.b(secret)
        self.server = server
        self.authport = authport
        self.acctport = acctport
        self.debug = debug
        reactor.listenUDP(0, self)

    def close(self):
        if self.transport is not None:
            self.transport.stopListening()
            self.transport = None

    def onError(self, err):
        log.err('Packet process error：%s' % str(err))
        reactor.callLater(0.01, self.close,)
        return err

    def onResult(self, resp):
        reactor.callLater(0.01, self.close,)
        return resp

    def onTimeout(self):
        if not self.deferrd.called:
            defer.timeout(self.deferrd)

    def sendAuth(self, **kwargs):
        timeout_sec = kwargs.pop('timeout',10) 
        User_Password = kwargs.pop("User-Password",None)
        CHAP_Password = kwargs.pop("CHAP-Password",None)
        CHAP_Challenge = kwargs.get("CHAP-Challenge")
        request = message.AuthMessage(dict=self.dict, secret=self.secret, **kwargs)
        if User_Password:
            request['User-Password'] = request.PwCrypt(User_Password)
        if CHAP_Password:
            if CHAP_Challenge: 
                request['CHAP-Challenge'] = CHAP_Challenge
            request['CHAP-Password'] = CHAP_Password

        if self.debug:
            log.msg("Send radius Auth Request to (%s:%s): %s" % (self.server, self.authport, request.format_str()))
        self.transport.write(request.RequestPacket(), (self.server, self.authport))
        self.deferrd = defer.Deferred()
        self.deferrd.addCallbacks(self.onResult,self.onError)
        reactor.callLater(timeout_sec, self.onTimeout,)
        return self.deferrd

    def sendAcct(self, **kwargs):
        timeout_sec = kwargs.pop('timeout',10) 
        request = message.AcctMessage(dict=self.dict, secret=self.secret, **kwargs)
        if self.debug:
            log.msg("Send radius Acct Request to (%s:%s): %s" % (self.server, self.acctport, request.format_str()))
        self.transport.write(request.RequestPacket(), (self.server, self.acctport))
        self.deferrd = defer.Deferred()
        self.deferrd.addCallbacks(self.onResult,self.onError)
        reactor.callLater(timeout_sec, self.onTimeout,)
        return self.deferrd

    def datagramReceived(self, datagram, (host, port)):
        try:
            response = packet.Packet(packet=datagram,dict=self.dict, secret=self.secret)
            if self.debug:
                log.msg("Received Radius Response from %s: %s" % ((host, port), message.format_packet_str(response)))
            self.deferrd.callback(response)
        except Exception as err:
            log.err('Invalid Response packet from %s: %s' % ((host, port), str(err)))
            self.deferrd.errback(err)


def send_auth(secret, dictionary, server, authport=1812, acctport=1813, debug=False, **kwargs):
    return RadiusClient(secret, dictionary, server, authport, acctport, debug).sendAuth(**kwargs)

def send_acct(secret, dictionary, server, authport=1812, acctport=1813, debug=False, **kwargs):
    return RadiusClient(secret, dictionary, server, authport, acctport, debug).sendAcct(**kwargs)












#!/usr/bin/env python
#coding=utf-8

import six
from twisted.python import log
from twisted.internet import protocol
from twisted.internet import reactor, defer
from txradius.radius import packet
from txradius.ext import ikuai
from txradius import message


class CoAClient(protocol.DatagramProtocol):
    
    def __init__(self, vendor_id, dictionary, nas_secret, nas_addr, coa_port=3799, debug=False):
        self.dictionary = dictionary
        self.secret = six.b(str(nas_secret))
        self.addr = nas_addr
        self.port = int(coa_port)
        self.vendor_id = int(vendor_id)
        self.debug=debug
        self.uport = reactor.listenUDP(0, self)

    def close(self):
        if self.transport is not None:
            self.transport.stopListening()
            self.transport = None

    def onError(self, err):
        log.err('Packet process error：%s' % str(err))
        reactor.callLater(0.01, self.close,)
        return err

    def onResult(self, resp):
        reactor.callLater(0.01, self.close,)
        return resp

    def onTimeout(self):
        if not self.deferrd.called:
            defer.timeout(self.deferrd)
        
    def sendDisconnect(self, **kwargs):
        timeout_sec = kwargs.pop('timeout',5) 
        coa_req = message.CoAMessage(
            code=packet.DisconnectRequest, dict=self.dictionary, secret=self.secret, **kwargs)   
        username = coa_req["User-Name"][0]
        if self.vendor_id == ikuai.VENDOR_ID:
            pkg = ikuai.create_dm_pkg(self.secret, username)
            if self.debug:
                log.msg("send ikuai radius Coa Request to (%s:%s) [username:%s]: %s"%(
                    self.addr, self.port, username, repr(pkg)))
            self.transport.write(pkg,(self.addr, self.port))
        else:
            if self.debug:
                log.msg("send radius Coa Request to (%s:%s) [username:%s] : %s"%(
                    self.addr, self.port, username, coa_req))
            self.transport.write(coa_req.RequestPacket(),(self.addr, self.port))
        self.deferrd = defer.Deferred()
        self.deferrd.addCallbacks(self.onResult,self.onError)
        reactor.callLater(timeout_sec, self.onTimeout,)
        return self.deferrd

    def datagramReceived(self, datagram, (host, port)):
        try:
            response = packet.Packet(packet=datagram)
            if self.debug:
                log.msg("Received Radius Response from (%s:%s): %s" % (host, port, repr(response)))
            self.deferrd.callback(response.code)
        except Exception as err:
            log.err('Invalid Response packet from %s: %s' % ((host, port), str(err)))
            self.deferrd.errback(err)


def disconnect(vendor_id, dictionary, nas_secret, nas_addr, coa_port=3799, debug=False, **kwargs):
    return CoAClient(vendor_id, dictionary, nas_secret, nas_addr, coa_port, debug).sendDisconnect(**kwargs)






#!/usr/bin/env python
#coding=utf-8

def parse_cisco(req):
    for attr in req:
        if attr not in 'Cisco-AVPair':
            continue
        attr_val = req[attr]
        if attr_val.startswith('client-mac-address'):
            mac_addr = attr_val[len("client-mac-address="):]
            mac_addr = mac_addr.replace('.','')
            _mac = (mac_addr[0:2],mac_addr[2:4],mac_addr[4:6],mac_addr[6:8],mac_addr[8:10],mac_addr[10:])
            req.client_mac =  ':'.join(_mac)
    return req


def parse_radback(req):
    mac_addr = req.get('Mac-Addr')
    if mac_addr:
        req.client_mac = mac_addr.replace('-',':')
    return req


def parse_zte(req):
    mac_addr = req.get('Calling-Station-Id')
    if mac_addr:
        mac_addr = mac_addr[12:] 
        _mac = (mac_addr[0:2],mac_addr[2:4],mac_addr[4:6],mac_addr[6:8],mac_addr[8:10],mac_addr[10:])
        req.client_mac =  ':'.join(_mac)
    return req

  
def parse_h3c(req):
    mac_addr = req.get('H3C-Ip-Host-Addr')
    if mac_addr and len(mac_addr) > 17:
        req.client_mac = mac_addr[:-17]
    else:
        req.client_mac = mac_addr

    return req


_parses = {
            '0' : parse_zte,
            '9' : parse_cisco,
            '2352' : parse_radback,
            '3902' : parse_zte,
            '25506' : parse_h3c
        }



def process(req):
    if req.vendor_id in _parses:
        _parses[req.vendor_id](req)
    return req






